<?php declare(strict_types=1);

namespace Convo\Core\Adapters\Fbm;

class FacebookMessengerServicePublisher extends \Convo\Core\Publish\AbstractServicePublisher
{
    /**
     * @var \Convo\Core\Adapters\Fbm\FacebookMessengerApiFactory
     */
    private $_facebookMessengerApiFactory;

    public function __construct( $logger, \Convo\Core\IAdminUser $user, $serviceId, $facebookMessengerApiFactory, $serviceDataProvider, $serviceReleaseManager)
	{
	    parent::__construct( $logger, $user, $serviceId, $serviceDataProvider, $serviceReleaseManager);
	    $this->_facebookMessengerApiFactory = $facebookMessengerApiFactory;
	}

	public function getPlatformId()
	{
		return 'facebook_messenger';
	}

	public function export()
	{
	    throw new \Exception( 'Not supported yet');
	}

	public function enable()
	{
	    parent::enable();
        $config            =   $this->_convoServiceDataProvider->getServicePlatformConfig( $this->_user, $this->_serviceId, 'develop');
        if (isset( $config[$this->getPlatformId()])) {
            $this->_updateFacebookMessengerBot();
        } else {
            throw new \Exception("Missing platform config [" . $this->getPlatformId());
        }
	}

	public function propagate()
    {
        $config            =   $this->_convoServiceDataProvider->getServicePlatformConfig( $this->_user, $this->_serviceId, 'develop');
        if (isset( $config[$this->getPlatformId()])) {
            $this->_updateFacebookMessengerBot();
            $this->_recordPropagation();
        } else {
            throw new \Exception("Missing platform config [" . $this->getPlatformId());
        }
    }

    public function getPropagateInfo()
    {
        $this->_logger->info("Accessing propagate info...");
        $data              =   parent::getPropagateInfo();
        $config            =   $this->_convoServiceDataProvider->getServicePlatformConfig( $this->_user, $this->_serviceId, 'develop');

        if ( !isset( $config[$this->getPlatformId()])) {
            $this->_logger->debug( 'No platform ['.$this->getPlatformId().'] config in service ['.$this->_serviceId.']. Exiting ... ');
            return $data;
        }

        $this->_logger->info(print_r($config, true) . " Accessing platform config");
        $platform_config   =   $config[$this->getPlatformId()];

        $this->_logger->debug( 'Got auto mode. Checking further ... ');

        $data['allowed'] = true;

        $workflow  =   $this->_convoServiceDataProvider->getServiceData( $this->_user, $this->_serviceId, 'develop');
        $meta      =   $this->_convoServiceDataProvider->getServiceMeta( $this->_user, $this->_serviceId, 'develop');

        if (isset($meta['release_mapping'][$this->getPlatformId()])) {
          $alias     =   $this->_serviceReleaseManager->getDevelopmentAlias( $this->_user, $this->_serviceId, $this->getPlatformId());
          $mapping   =   $meta['release_mapping'][$this->getPlatformId()][$alias];

          if ( !isset( $mapping['time_propagated']) || empty( $mapping['time_propagated'])) {
            $this->_logger->debug( 'Never propagated ');
            $data['available'] = true;
          } else {
            if ( $mapping['time_propagated'] < $platform_config['time_updated']) {
              $this->_logger->debug( 'Config changed');
              $data['available'] = true;
            }

            if ( isset( $mapping['time_updated']) && ($mapping['time_propagated'] < $mapping['time_updated'])) {
              $this->_logger->debug( 'Mapping changed');
              $data['available'] = true;
            }

            if ( $mapping['time_propagated'] < $workflow['intents_time_updated']) {
              $this->_logger->debug( 'Intents model changed');
              $data['available'] = true;
            }
          }
        }

        return $data;
    }

    private function _updateFacebookMessengerBot() {
        $facebookMessengerApi = $this->_facebookMessengerApiFactory->getApi($this->_user, $this->_serviceId, $this->_convoServiceDataProvider);
        $url = $this->_serviceReleaseManager->getWebhookUrl($this->_user, $this->_serviceId, $this->getPlatformId());
        $facebookMessengerApi->callSubscriptionsApi($url);
        $facebookMessengerApi->callSubscribedApps();
        $facebookMessengerApi->callMessengerProfileApi();
    }

    public function delete(array &$report)
    {
        // TODO: Implement delete() method.
    }
}
