<?php

/** @var \Psr\Container\ContainerInterface $container */
/** @var \Psr\Log\LoggerInterface $logger */

if (!file_exists('../../common/__config.php')) {
	die('No configuration file!');
}

require_once('../../library/autoload.php');
require_once('../../common/__config.php');

// MAINTENACE MODE
if (!defined('UTIL_PUBLIC_MAINTENACE_MODE')) {
	define('UTIL_PUBLIC_MAINTENACE_MODE', false);
}

if (UTIL_PUBLIC_MAINTENACE_MODE) {
	header("HTTP/1.0 503 Service unavailable");
	header('Content-Type: application/json');
	die('{"message":"Service unavailable"}');
}


// DEPENDENCY INJECTION
$builder = new DI\ContainerBuilder();
$builder->addDefinitions('../../common/di-core.php');
$builder->addDefinitions('../../common/di-data.php');
$builder->addDefinitions('di-client.php');

$container = $builder->build();



/** @var \Psr\Log\LoggerInterface $logger */
$logger         =   $container->get('logger');

// REQUEST
/** @var \Psr\Http\Message\ServerRequestInterface $request */
$request        =   \GuzzleHttp\Psr7\ServerRequest::fromGlobals();

$start = microtime( true);

try {

    // ROUTING
    $info	=	new \Convo\Core\Rest\RequestInfo( $request, 'rest_public');

    if ( $info->startsWith( 'convo')) {
        $handler    =   new \Convo\Core\Adapters\PublicRestApi( $logger, $container);
    } else if ( $info->startsWith( 'proto/v1/users')) {
        $handler    =   $container->get( '\Convo\Proto\ServiceUsersRestHandler');
    } else if ( $info->startsWith( 'proto/v1/oauth') || $info->startsWith( 'proto/v1/token')) {
        $handler    =   $container->get( '\Convo\Proto\OAuthRestHandler');
    } else {
        throw new \Convo\Core\Rest\NotFoundException( 'Could not map ['.$info.']');
    }

    // MIDDLEWARES
    $middlewares    =   require_once('middlewares.php');

    // APP ITSELF
    $app            =   new \Convo\Core\Util\RestApp( $logger, $container, $handler, $middlewares);
    $response       =   $app->handle( $request);
    $app->writeResponse( $response);

} catch ( Convo\Core\Rest\NotFoundException $e) {
    $logger->warning( $e);
    header("HTTP/1.0 404 Not Found");
    header('Content-Type: application/json');
    die('{"message":"Requested resource was not found"}');
} catch ( Throwable $e) {
    $logger->critical( $e);
    header("HTTP/1.0 500 Error");
    header('Content-Type: application/json');
    die('{"message":"Unexpected error"}');
} finally {
    $time_elapsed_us = microtime( true) - $start;
    $logger->debug('DONE in ' . ($time_elapsed_us * 1000) . ' ms');
}

